'use strict'

// Concept - closure. Inner function code carries around its enclosing
// scope. The variable 'name', local to sayHello2 is still accessible
// after the function has completed.

const sayHello = function (name) {
  const text = 'Hello ' + name
  const say = function print () { console.log(text) }
  return say
}

const greet = sayHello('Thabo')
greet()

// Output: Hello Thabo

// ---------------------slide---------------------------

// Closure concept - all variables in the outer function form part of the closure
function sayAlice () {
  const sayAlert = function greeting () { console.log(alice) }
  // Local variable is hoisted and ends up within closure
  const alice = 'Hello Alice'
  return sayAlert
}
sayAlice()() // immediately invoke the returned function expression

// Output: Hello Alice

// ---------------------slide---------------------------

// Closure concept - the scope closed over has the state resulting from
// the completion of the outer function (like keeping a stack frame in memory)

function say5 () {
  let num = 5
  const say = function () { console.log(num) }
  num++
  return say
}

say5()() // immediately invoke the returned function expression

// Output: 6

// ---------------------slide---------------------------

// Closure concept: each call to the outer function
// results in a separate closure (stack frame)

const namer = function (name) {
  return function (obj) {
    obj.name = name
    console.log(obj)
  }
}

const anObj = { groupNum: 12 }

const nameFrancis = namer('Francis')
const nameRyan = namer('Ryan')

nameFrancis(anObj) // name set to Francis
nameRyan(anObj) // name changed to Ryan

// What is the output?

// Output:
// { groupNum: 12, name: 'Francis' }
// { groupNum: 12, name: 'Ryan' }

// -------------------slide-----------------------------

// Closure concept: All inner functions share access to the same closure

let gPrintNumber, gIncreaseNumber, gSetNumber // globals

function setupSomeGlobals () {
  // Local variable that ends up within closure
  let num = 5
  // Store some references to functions as global variables
  gPrintNumber = function print () { console.log(num) }
  gIncreaseNumber = function increase () { num++ }
  gSetNumber = function assign (x) { num = x }
}

setupSomeGlobals()

gPrintNumber()
gIncreaseNumber()
gPrintNumber()
gSetNumber(44)
gPrintNumber()

// Output:
// 5
// 6
// 44

// ------------------------------------------------
